using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.IO;
using System.Threading;


namespace slideshow_0_3
{
	/// <summary>
	/// Slide Show 0.32 by Neil Moomey, www.neilmoomey.com, 9-16-04
	/// I created this program for my photography club.  We needed a simple and foolproof way to 
	/// display images full screen at our meetings. The program launches a File Chooser. After 
	/// you select the first image to display it goes to full screen mode with a black background 
	/// and no border. I have found this program works great for my initial edit of images so I have 
	/// added some features I find useful.  Make sure your images are not protected if you wish to 
	/// use the rotate and delete features.
	///
	/// Arrow keys: Navigation
	/// Page Down/Up: Navigation
	/// Home: Go to first image
	/// End: Go to last image
	/// Space bar: Read detailed EXIF data
	/// F1: toggle on/off footer information.
	/// F2: Toggle on/off Auto-Rotate feature based on EXIF image orientation data
	/// Delete:  Move file to a folder called _Delete
	/// c: Copy image to a folder called _Copy
	/// r: Rotate right and lossless save	
	/// l: Rotate left and lossless save 
	/// Esc: Exit the program.
	/// Note:  Lossless rotation only if the width and height are both multiples of 16 pixels such as most digital 
	/// cameras produce. See Microsoft .Net documentation for details
	/// Also the auto-rotate feature will only rotate an image from landscape to portrait orientation.  This ensures it 
	/// only rotates the image once.  The EXIF orientation data is unaltered.
	/// </summary>

	public class Form1 : System.Windows.Forms.Form
	{

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Drawing.Size screenSize;
		private String currentFile;
		private int pointer;
		private ArrayList fileList;
		System.Collections.IEnumerator myEnumerator;
		String directoryName, directoryParent;
		private Button button;
		private bool panelFlag, footerFlag, rotateFlag;
		private float w,h;


		public Form1()
		{
			InitializeComponent();
			FormBorderStyle = FormBorderStyle.None;
			pointer = 0;
			panelFlag=false;
			footerFlag=false;
			rotateFlag=false;
			currentFile="";
			this.SuspendLayout();

			//System.Resources.ResourceManager resources = new System.Resources.ResourceManager (typeof(Form1));
			//this.Icon = (System.Drawing.Icon) resources.GetObject ("$this.Icon");
			//System.Drawing.Icon ico = new System.Drawing.Icon("MISC41.ICO");
			//this.Icon = ico; 

			// What is screen size? (1024x768 most likely) use: screenSize.Width, screenSize.Height
			screenSize = System.Windows.Forms.Screen.PrimaryScreen.Bounds.Size;

			this.AutoScaleBaseSize = new System.Drawing.Size(0, 0);
			this.ClientSize = new System.Drawing.Size(screenSize.Width, screenSize.Height);
			this.Name = "Slide Show";
			this.ResumeLayout(false);


			// The first step is to open a File Chooser
			currentFile = this.BrowseOpen(); //String
			
			//Cursor.Hide();

	        try
        	{
				fileList = GetFileList(currentFile); //ArrayList of Strings
				pointer = fileList.IndexOf(currentFile); // int of the image postion in the ArrayList
				Console.WriteLine("currentFile at startup = " + currentFile);
				Console.WriteLine("fileList[{0}]= {1}", pointer, fileList[pointer]);
        	}
       	 	catch (Exception e)
        	{
            	Console.WriteLine("This process failed: {0}", e.ToString());
				Environment.Exit(-1);
        	}

			this.KeyUp += new KeyEventHandler(keyHandler);
		}


		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

 		// Draw Image to Screen.  Forms.Control.OnPaint
 		// Use Invalidate() to repaint.
 		protected override void OnPaint(PaintEventArgs e)
		{
			int displayWidth,displayHeight,x,y;
			String text, fNumber, exposureTime, exposureBiasValue, ISO, focalLength;
			PointF drawPoint;
			Graphics g = e.Graphics;
			SolidBrush brush;
			Bitmap bmp;

			// Give focus to screen
			this.Activate();

			currentFile=fileList[pointer].ToString();
						
			// Auto rotate if the flag is on and it's a landscape image
			if (rotateFlag==true) 
			{
				bmp = new Bitmap(currentFile);
				displayWidth=bmp.Width;
				displayHeight=bmp.Height;
				ExifReader r = new ExifReader(currentFile);
				String orientation = r.GetOrientation();
				Console.WriteLine("Orientation: " + r.GetOrientation());
				r.MyDispose(); // Release handle so we can rotate it
				bmp.Dispose(); // Release handle so we can rotate it
				if (displayWidth>displayHeight && orientation=="Rotate Right") Rotate90(currentFile);
				if (displayWidth>displayHeight && orientation=="Rotate Left") Rotate270(currentFile);
			}			

			bmp = new Bitmap(currentFile);
			displayWidth=bmp.Width;
			displayHeight=bmp.Height;
			
			
			// Must use float for h/w ratio because int will return 0
			// Will use w, h later so make it a class variable
			w=bmp.Width;
			h=bmp.Height;
			System.Console.WriteLine("w1 = "+ displayWidth + " h1 = " + displayHeight);			

			// Determine the width and height to display full screen
			if (bmp.Width*bmp.Height != 0)
			{
				if (bmp.Width> screenSize.Width)
				{
					displayHeight = (int)((h/w)*screenSize.Width);
					displayWidth=screenSize.Width;
				}
				if (displayHeight > screenSize.Height)
				{
					displayWidth = (int)((w/h)*screenSize.Height);
					displayHeight=screenSize.Height;
				}
			}
			

			System.Console.WriteLine("w2 = "+ displayWidth + " h2 = " + displayHeight);

			// Center image on screen
			x=0;
			y=0;
			if (displayWidth<screenSize.Width) x = (screenSize.Width/2)-(displayWidth/2);
			if (displayHeight<screenSize.Height) y = (screenSize.Height/2)-(displayHeight/2);
			Rectangle r1 = new Rectangle(x,y,displayWidth,displayHeight);
			Rectangle r2 = new Rectangle(0,0,bmp.Width, bmp.Height);

			// Draw the image
			g.DrawImage(bmp,r1,r2,GraphicsUnit.Pixel);

			// Add footer EXIF data
			if (footerFlag) 
			{
				// Fetch EXIF data using ExifReader Class in this project
				ExifReader r = new ExifReader(currentFile);
				fNumber =  r.GetFNumber();
				exposureTime = r.GetExposureTime();
				if (exposureTime.Length==1) exposureTime += " Sec";
				if (r.GetExposureMode() != "Manual Exposure") exposureBiasValue = r.GetExposureBiasValue();
					else exposureBiasValue="";
				ISO = r.GetISO();
				focalLength = r.GetFocalLength();
				brush = new SolidBrush(Color.Gray);

				// Vertical EXIF images
				//if ((displayHeight > displayWidth) && (displayHeight > screenSize.Height - 40))
				if ( ((float)displayHeight/(float)displayWidth) > 0.9  && screenSize.Width - displayWidth > 300 )
				{
						text = "";
						if (fNumber != "") text += "F" +fNumber + "\n";
						if (exposureTime != "") text += exposureTime + "\n"; 
						if (exposureBiasValue != "") text += exposureBiasValue + "\n";
						if (ISO != "") text += "ISO" + ISO + "\n";
						if (focalLength != "") text += focalLength + "mm\n";	
						text += Path.GetFileName(currentFile) + "\n";
						drawPoint = new PointF(10F,(float)(screenSize.Height-205));//165
				}
				else
				{
					// Horizontal EXIF images
					text = "";
					if (fNumber != "") text += "F" +fNumber + ", ";
					if (exposureTime != "") text += exposureTime + ", "; 
					if (exposureBiasValue != "") text += exposureBiasValue + ", ";
					if (ISO != "") text += "ISO" + ISO + ", ";
					if (focalLength != "") text += focalLength + "mm , ";	
					text += Path.GetFileName(currentFile);
					drawPoint = new PointF(10F,(float)(screenSize.Height-40));  //PointF 
					if (displayHeight > screenSize.Height - 40) brush = new SolidBrush(Color.White);
				}

				
				//Rectangle r3 = new Rectangle(0,displayHeight-10,displayWidth,40);
				if (displayHeight > screenSize.Height - 40)
					g.DrawString(text,new Font("Arial",22),brush,drawPoint);
				else 
					g.DrawString(text,new Font("Arial",22),brush,drawPoint);

				r.MyDispose();
			}
			
			// Release resources so we can delete file if we wish
			g.Dispose();
			bmp.Dispose();
		}


		public ArrayList GetFileList(String source)
		{
			String ext;
			int length;
			ArrayList imageNames = new ArrayList();
			myEnumerator = imageNames.GetEnumerator();

			DirectoryInfo directoryInfo = Directory.GetParent(source);
			directoryName = directoryInfo.FullName.ToString();  //String
			length=directoryName.LastIndexOf("\\");
			directoryParent = directoryName.Substring(0,length);  //String
			System.Console.WriteLine("directoryName = "+ directoryName);
			System.Console.WriteLine("directoryParent = "+ directoryParent);


			// Create an array representing the files in this directory.
			FileInfo[] fi = directoryInfo.GetFiles();
	        foreach (FileInfo fiTemp in fi)
	        {
            	//Console.WriteLine("ext = " + fiTemp.Extension);
            	ext = fiTemp.Extension.ToLower();
            	if ((ext==".jpg") || (ext==".jpeg") || (ext==".gif") || (ext==".png") || (ext==".tif") || (ext==".tiff") || (ext==".bmp"))
            	{
					imageNames.Add(fiTemp.FullName);
					Console.WriteLine("Name = " + fiTemp.FullName);

				}
				// Sort files in alphabetical order
				imageNames.Sort();
    		}
			return imageNames;
		}

		private String BrowseOpen()
		{
			// File Chooser to select the first image to be displayed
			OpenFileDialog fdlg = new OpenFileDialog();
			fdlg.Title = "Please Select the First Image to Display." ;
			fdlg.Filter = "Image Files (JPEG,GIF,BMP,PNG,TIFF)|*.jpg;*.jpeg;*.gif;*.bmp;*.png;*.tif;*.tiff|JPEG Files(*.jpg;*.jpeg)|*.jpg;*.jpeg|GIF Files(*.gif)|*.gif|BMP Files(*.bmp)|*.bmp|PNG Files(*.png)|*.png|TIFF Files(*.tif;*tiff)|*.tif;*.tiff";

			fdlg.FilterIndex = 1 ;
			//fdlg.RestoreDirectory = true ;
			if(fdlg.ShowDialog() == DialogResult.OK)
			{
				return fdlg.FileName;
			}
			else
			{
				return null;
			}
		}

		// Handle Key Events
		private void keyHandler(Object sender, KeyEventArgs e)  {
			if ( e.KeyCode == Keys.Escape )
			{
				if (panelFlag) 
				{
					this.Controls.Remove(button);
					panelFlag=false;
					Invalidate();
				}
				else 
					Environment.Exit(-1);
			}

			if ( e.KeyCode == Keys.Down || e.KeyCode == Keys.Right || e.KeyCode == Keys.PageDown)
			{
				//Console.WriteLine("Down1  Item =" + currentFile);
				if(pointer<fileList.Count-1)
				{
					pointer++;
					// This calls the OnPaint method which repaints the screen
					Invalidate();
				}
				Console.WriteLine("ArrayList at pointer "+ pointer + " = " + fileList[pointer]);
			}

			if ( e.KeyCode == Keys.Up || e.KeyCode == Keys.Left || e.KeyCode == Keys.PageUp)
			{
				//Console.WriteLine("Down1  Item =" + currentFile);
				if(pointer>0) {
					pointer--;
					// This calls the OnPaint method which repaints the screen
					Invalidate();
				}
				Console.WriteLine("ArrayList at pointer "+ pointer + " = " + fileList[pointer]);
			}

			// Jump to the beginning
			if ( e.KeyCode == Keys.Home )
			{
				pointer=0;
				Invalidate();
			}

			// Jump to the end
			if ( e.KeyCode == Keys.End )
			{
				pointer=fileList.Count-1;
				Invalidate();
			}

			// Jump 5 images at a time
			if ( e.KeyCode == Keys.NumPad5)
			{
				pointer=fileList.Count-5;
				Invalidate();
			}			

			if ( e.KeyCode == Keys.F1 )
			{
				if (footerFlag==true) footerFlag=false;
				else footerFlag=true;
				Invalidate();
			}
			
			if ( e.KeyCode == Keys.F2 )
			{
				if (rotateFlag==true) rotateFlag=false;
				else rotateFlag=true;
				Invalidate();
			}			

			if ( e.KeyCode == Keys.Delete )
			{	
				try 
				{
					String currentFile2, deleteDir, newFile;

					deleteDir = directoryParent + "\\_Delete";
					if(!Directory.Exists(deleteDir)) Directory.CreateDirectory(deleteDir);
					newFile = deleteDir + "\\" + Path.GetFileName(currentFile);

					currentFile2=currentFile;

					// Advance position if possible
					if(pointer<fileList.Count-1) fileList.RemoveAt(pointer);
					else 
						if(pointer==fileList.Count-1) 
					{
						pointer--;
						fileList.RemoveAt(fileList.Count-1);
					}
					// If there are no more photos left exit the program
					if (fileList.Count==0) Environment.Exit(-1);
					Invalidate();

					// Ensure that the target does not exist in the Delete Directory
					if (File.Exists(newFile)) File.Delete(newFile);

					Console.WriteLine("***Move from = " + currentFile2 + " to = " + newFile);
					File.Move(currentFile2, newFile);

				}
				catch (Exception e2) 
				{
					Console.WriteLine("Delete process failed: {0}", e2.ToString());
				}
			}
			
			// Copy file to Copy folder
			if ( e.KeyCode == Keys.C )
			{	
				try 
				{
					String currentFile2, copyDir, newFile;

					copyDir = directoryParent + "\\_Copy";
					if(!Directory.Exists(copyDir)) Directory.CreateDirectory(copyDir);
					newFile = copyDir + "\\" + Path.GetFileName(currentFile);

					currentFile2=currentFile;

					// Ensure that the target does not exist in the _Copy Directory
					if (File.Exists(newFile)) File.Delete(newFile);
					
					Console.WriteLine("***Move from = " + currentFile2 + " to = " + newFile);
					File.Copy(currentFile2, newFile);

				}
				catch (Exception e2) 
				{
					Console.WriteLine("Delete process failed: {0}", e2.ToString());
				}
			}
			
			if ( e.KeyCode == Keys.Space )
			{
				if (panelFlag) 
				{
					this.Controls.Remove(button);
					panelFlag=false;
				}
				else 
				{
					panelFlag=true;
					ExifReader r = new ExifReader(currentFile);
					String fileName = Path.GetFileName(currentFile);
					String text="\n";
					
					if (r.GetFNumber()!="") text+="   FStop: F" + r.GetFNumber() + "\n";
					if (r.GetExposureTime()!="") text+="   Shutter Speed: " + r.GetExposureTime() + "\n";
					if (r.GetISO()!="") text+="   ISO: " + r.GetISO() + "\n";
					if (r.GetExposureBiasValue()!="") text+="   Exposure Compensation: " + r.GetExposureBiasValue() + "\n";
					if (r.GetExposureMode()!="") text+="   Exposure Mode: " + r.GetExposureMode() + "\n";
					if (r.GetExposureProgram()!="") text+="   Exposure Program: " + r.GetExposureProgram() + "\n";
					if (r.GetMeteringMode()!="") text+="   Metering Mode: " + r.GetMeteringMode() + "\n";
					if (r.GetFlash()!="") text+="   Flash: " + r.GetFlash() + "\n";
					if (r.GetFocalLength()!="") text+="   Focal Length: " + r.GetFocalLength() + "\n";
					if (r.GetWhiteBalance()!="") text+="   White Balance: " + r.GetWhiteBalance() + "\n"; 
					if (r.GetMake()!="") text+="   Make: " + r.GetMake() + "\n"; 
					if (r.GetModel()!="") text+="   Model: " + r.GetModel() + "\n"; 
					if (r.GetCompression()!="") text+="   Compression: " + r.GetCompression() + "\n"; 
					if (r.GetArtist()!="") text+="   Artist: " + r.GetArtist() + "\n";
					if (r.GetSoftware()!="") text+="   Software: " + r.GetSoftware() + "\n";
					if (r.GetDateTimeOriginal()!="") text+="   DateTimeOriginal: " + r.GetDateTimeOriginal() + "\n";
					text+="   Width: " + w.ToString() + " pixels\n";
					text+="   Height: " + h.ToString() + " pixels\n";
					if (r.GetImageUniqueID()!="") text+="   ImageUniqueID: " + r.GetImageUniqueID() + "\n";
					text+="   Filename: " + fileName + "\n";

					button = new Button();
					this.button.Click += new System.EventHandler(this.button_Click);

					button.Text = text;
					Font font = new Font("Arial",22);
					button.Font=font;
					button.BackColor=Color.Black;
					button.ForeColor=Color.Gray;
					button.TextAlign = ContentAlignment.MiddleLeft;
					button.Size = new Size(screenSize.Width, screenSize.Height);

					this.Controls.Add(button);
				}
			}

			if ( e.KeyCode == Keys.RButton )
			{
				MessageBox.Show("Hello");
				Invalidate();
			}
			
			if ( e.KeyCode == Keys.R )
			{	
				Rotate90(currentFile);
				Invalidate();
			}

			if ( e.KeyCode == Keys.L )
			{	
				Rotate270(currentFile);
				Invalidate();
			}			
		}

		private void button_Click (object sender, System.EventArgs e)
		{
			this.Controls.Remove(button);
			panelFlag=false;
			Invalidate();
		}

		public static void Rotate90(string fileName)
		{ 
			Image Pic; 
			string FileNameTemp; 
			Encoder Enc = Encoder.Transformation; 
			EncoderParameters EncParms = new EncoderParameters(1); 
			EncoderParameter EncParm; 
			ImageCodecInfo CodecInfo = GetEncoderInfo("image/jpeg"); 
		 
			// load the image to change 
			Pic = Image.FromFile(fileName); 
		 
			// we cannot store in the same image, so use a temporary image instead 
			FileNameTemp = fileName + ".temp"; 
		
			// for rewriting without recompression we must rotate the image 90 degrees
			EncParm = new EncoderParameter(Enc,(long)EncoderValue.TransformRotate90); 
			EncParms.Param[0] = EncParm; 
		
			// now write the rotated image with new description 
			Pic.Save(FileNameTemp,CodecInfo,EncParms); 
			Pic.Dispose(); 
			Pic = null; 
		
			// delete the original file, will be replaced later 
			System.IO.File.Delete(fileName); 
			System.IO.File.Move(FileNameTemp, fileName); 
		}
		
		public static void Rotate270(string fileName)
		{ 
			Image Pic; 
			string FileNameTemp; 
			Encoder Enc = Encoder.Transformation; 
			EncoderParameters EncParms = new EncoderParameters(1); 
			EncoderParameter EncParm; 
			ImageCodecInfo CodecInfo = GetEncoderInfo("image/jpeg"); 
		 
			// load the image to change 
			Pic = Image.FromFile(fileName); 
		 
			// we cannot store in the same image, so use a temporary image instead 
			FileNameTemp = fileName + ".temp"; 
		
			// for rewriting without recompression we must rotate the image 90 degrees
			EncParm = new EncoderParameter(Enc,(long)EncoderValue.TransformRotate270); 
			EncParms.Param[0] = EncParm; 
		
			// now write the rotated image with new description 
			Pic.Save(FileNameTemp,CodecInfo,EncParms); 
			Pic.Dispose(); 
			Pic = null; 
		
			// delete the original file, will be replaced later 
			System.IO.File.Delete(fileName); 
			System.IO.File.Move(FileNameTemp, fileName); 
		}
		
			

				private static ImageCodecInfo GetEncoderInfo(String mimeType)
		{
			int j;
			ImageCodecInfo[] encoders;
			encoders = ImageCodecInfo.GetImageEncoders();
			for(j = 0; j < encoders.Length; ++j)
			{
				if(encoders[j].MimeType == mimeType)
					return encoders[j];
			} 
			return null;
		}
		
		public static void PrintValues( IEnumerable myList )
		{
			System.Collections.IEnumerator myEnumerator = myList.GetEnumerator();
			int i=0;
			while ( myEnumerator.MoveNext() )
			{
				i++;
				Console.WriteLine(i + " = " + myEnumerator.Current );

			}
		}

		private void InitializeComponent()
		{
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.BackColor = System.Drawing.Color.Black;
			this.ClientSize = new System.Drawing.Size(292, 273);
			this.Name = "Form1";
			this.Text = "Slide Show";
			this.WindowState = System.Windows.Forms.FormWindowState.Maximized;
			this.Load += new System.EventHandler(this.Form1_Load);

		}

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main()
		{
			Application.Run(new Form1());
		}

		private void Form1_Load(object sender, System.EventArgs e)
		{
		
		}


	}
}
